'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var isEqual = require('fast-deep-equal/react');
var React = require('react');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var isEqual__default = /*#__PURE__*/_interopDefaultLegacy(isEqual);
var React__default = /*#__PURE__*/_interopDefaultLegacy(React);

function useDeepCompareCallback(fn, deps) {
    const signal = React.useRef(0);
    const prevDeps = React.useRef(deps);
    if (!isEqual__default["default"](prevDeps.current, deps)) {
        signal.current += 1;
    }
    prevDeps.current = deps;
    return React.useCallback(fn, [signal.current]);
}

function useDeepCompareEffect(fn, deps) {
    const signal = React.useRef(0);
    const prevDeps = React.useRef(deps);
    if (!isEqual__default["default"](prevDeps.current, deps)) {
        signal.current += 1;
    }
    prevDeps.current = deps;
    React.useEffect(fn, [signal.current]);
}

function useDeepCompareMemo(fn, deps) {
    const signal = React.useRef(0);
    const prevDeps = React.useRef(deps);
    if (!isEqual__default["default"](prevDeps.current, deps)) {
        signal.current += 1;
    }
    prevDeps.current = deps;
    return React.useMemo(fn, [signal.current]);
}

function useEventCallback(handler) {
    const handlerRef = React__default["default"].useRef(handler);
    React__default["default"].useLayoutEffect(() => {
        handlerRef.current = handler;
    });
    return React__default["default"].useCallback((...args) => {
        const handle = handlerRef.current;
        return handle(...args);
    }, []);
}

function useInterval(callback, duration) {
    const callbackRef = React.useRef(callback);
    React.useEffect(() => {
        callbackRef.current = callback;
    }, [callback]);
    React.useEffect(() => {
        const tick = () => {
            if (callbackRef.current === undefined)
                return;
            callbackRef.current();
        };
        const id = setInterval(tick, duration);
        return () => clearInterval(id);
    }, [duration]);
}

function usePreviousState(value) {
    const ref = React.useRef(value);
    React.useEffect(() => {
        ref.current = value;
    }, [value]);
    return ref.current;
}

function useReactiveRef(value) {
    const ref = React__default["default"].useRef(value);
    ref.current = value;
    return ref;
}

function useSyncState(initialState) {
    const [state, setState] = React__default["default"].useState(initialState);
    React__default["default"].useEffect(() => setState(initialState), [initialState]);
    return [state, setState];
}

exports.useDeepCompareCallback = useDeepCompareCallback;
exports.useDeepCompareEffect = useDeepCompareEffect;
exports.useDeepCompareMemo = useDeepCompareMemo;
exports.useEventCallback = useEventCallback;
exports.useInterval = useInterval;
exports.usePreviousState = usePreviousState;
exports.useReactiveRef = useReactiveRef;
exports.useSyncState = useSyncState;
